package org.nci.dts.client;

import com.apelon.apelonserver.client.ServerConnection;
import com.apelon.dts.client.DTSException;
/**
 * <p>ConceptHistoryServerImpl is based on the ConceptHistoryServer.
 * <br>A ConceptHistoryServerImpl class does two thing:
 * <li> Connects the ConceptHistoryServer with a socket server or directly with an
 *      oracle database according to the server connection type.
 * <li> Sets DTD file.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Apelon Inc.</p>
 * @author Apelon Inc.
 * @version DTS 3.0
 */

final class ConceptHistoryServerImpl extends ConceptHistoryServer
{
        private ServerConnection srvr;

        /**
        * Constructs a new ConceptHistoryServerImpl.
        * <br>Depending on the type of ServerConnection object passed in the
        * argument, the server may be local(JDBC), socket, or another implementation.
        *
        * @param         sc              the type of server connection.
        * @exception     DTSException    error
        *
        * @since         DTS 2.4
        */
        ConceptHistoryServerImpl(ServerConnection sc) throws DTSException
        {
          this(sc, true);
        }

        /**
        * Constructs a new ConceptHistoryServerImpl.
        * <br>Depending on the type of ServerConnection object passed in the
        * argument, the server may be local(JDBC), socket, or another implementation.
        * <br>second argument is a boolean value for xml validation.
        *
        * @param         sc              the type of server connection.
        * @param         isValidating    a boolean value. It is true if xml needs to
        *                                be validated, otherwise it is false.
        * @exception     DTSException    error
        *
        * @since         DTS 2.4
        */
        ConceptHistoryServerImpl(ServerConnection sc, boolean isValidating) throws DTSException
        {
                srvr = sc;
                initializeDtds(org.nci.dts.dtd.DTD.QUERY,
                               org.nci.dts.dtd.DTD.class,
                               org.nci.dts.dtd.DTD.RESULT,
                               org.nci.dts.dtd.DTD.RESULT_FILE,
                               isValidating);
        }

        /**
        * Gets the server connection type.
        *
        * @return        the type of the server connection.
        *
        * @since         DTS 2.4
        */
        protected ServerConnection getServerConnection()
        {
                return srvr;
        }
}