package org.nci.dts.client;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import com.apelon.apelonserver.client.BasicQuery;
import com.apelon.apelonserver.client.ServerConnection;
import com.apelon.apelonserver.client.ApelonException;
import com.apelon.common.xml.XML;
import com.apelon.common.xml.XMLException;
import com.apelon.dts.client.DTSException;
/**
 * <p>A DbInfoServer provides common methods for getting database information.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Apelon Inc.</p>
 * @author Apelon Inc.
 * @version DTS 3.0
 */

public abstract class DbInfoServer extends BasicQuery
{
        /**
        * Creates an instance of DbInfoServer.
        * <br>Depending on the type of ServerConnection object passed in the
        * argument, the server may be local (JDBC), socket, or another implementation.
        *
        * @param         sc              server connectin type
        * @return        the created DbInfoServer
        * @exception     DTSException    error
        *
        * @see           com.apelon.apelonserver.client.ServerConnectionJDBC
        * @see           com.apelon.apelonserver.client.ServerConnectionSocket
        * @since         DTS 2.4
        */
        public static DbInfoServer createInstance(ServerConnection sc)
            throws DTSException
        {
                DbInfoServer new_srvr = null;
                if( sc == null )
                {
                        throw new DTSException("DbInfoServer.createInstance() called with null argument");
                }
                new_srvr = new DbInfoServerImpl(sc);
                new_srvr.setVersion("DTS:021");
                return new_srvr;
        }
        /**
        * Creates an instance of DbInfoServer.
        * <br>Depending on the type of ServerConnection object passed in the
        * argument, the server may be local (JDBC), socket, or another implementation.
        * <br>The second argument is a boolean value for xml validation.
        *
        * @param         sc              server connectin type
        * @param         isValidating    a boolean value. It is true if xml needs to
        *                                be validated, otherwise it is false.
        * @return        the created DbInfoServer
        * @exception     DTSException    error
        *
        * @see           com.apelon.apelonserver.client.ServerConnectionJDBC
        * @see           com.apelon.apelonserver.client.ServerConnectionSocket
        * @since         DTS 2.4
        */
        public static DbInfoServer createInstance(ServerConnection sc, boolean isValidating)
            throws DTSException
        {
                DbInfoServer new_srvr = null;
                if( sc == null )
                {
                        throw new DTSException("DbInfoServer.createInstance() called with null argument");
                }
                new_srvr = new DbInfoServerImpl(sc, isValidating);
                new_srvr.setVersion("DTS:021");
                return new_srvr;
        }

        /**
        * Retrieves database infomation for the specified database name.
        *
        * @param        dbname    - the database name.
        * @return       DbInfo    - A DbInfo object for the specified database name.
        * @exception    DTSException      error
        *
        * @since        DTS 2.4
        */
        public DbInfo getDbInfo(String dbname)
            throws DTSException
        {
                String query = "";
                query += XML.createProlog("getDbInfo", org.nci.dts.dtd.DTD.QUERY, false);
                query += "<getDbInfo>";
                query += "<dbInfo value=\"name\">";
                query += XML.asValue(dbname);
                query += "</dbInfo>";
                query += "</getDbInfo>";

                Document doc;
                try {
                        doc = executeQuery(query);
                } catch (ApelonException ex) {
                        throw new DTSException("Error executing XML query.", ex);
                }
                return DbInfo.createDbInfo(doc);
        }

        /**
        * Retrieves all database infomation.
        *
        * @return       DbInfo[]    - An array of DbInfo object.
        * @exception    DTSException      error
        *
        * @since        DTS 2.4
        */
        public DbInfo[] getDbInfos()
            throws DTSException
        {
                String query = "";
                query += XML.createProlog("getDbInfo", org.nci.dts.dtd.DTD.QUERY, false);
                query += "<getDbInfo>";
                query += "<dbInfo value=\"all\"/>";
                query += "</getDbInfo>";

                Document doc;
                try {
                        doc = executeQuery(query);
                } catch (ApelonException ex) {
                        throw new DTSException("Error executing XML query.", ex);
                }
                return DbInfo.createDbInfos(doc);
        }
}