package org.nci.dts.client;

import com.apelon.apelonserver.client.ServerConnection;
import com.apelon.dts.client.DTSException;
 /**
 * <p>DbInfoServerImpl is based on the DbInfoServer.
 * <br>A DbInfoServerImpl class does two thing:
 * <li> Connects the DbInfoServer with a socket server or directly with an
 *      oracle database according to the server connection type.
 * <li> Sets DTD file.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Apelon Inc.</p>
 * @author Apelon Inc.
 * @version DTS 3.0
 */

final class DbInfoServerImpl extends DbInfoServer
{
	private ServerConnection srvr;
        /**
         * Constructs a new DbInfoServerImpl.
         * <br>Depending on the type of ServerConnection object passed in the
         * argument, the server may be local(JDBC), socket, or another implementation.
         *
         * @param         sc              the type of server connection.
         * @exception     DTSException    error
         *
         * @since         DTS 2.4
         */
	DbInfoServerImpl(ServerConnection sc) throws DTSException
	{
          this(sc, true);
	}

        /**
         * Constructs a new DbInfoServerImpl.
         * <br>Depending on the type of ServerConnection object passed in the
         * argument, the server may be local(JDBC), socket, or another implementation.
         * <br>The second argument is a boolean value for xml validation.
         *
         * @param         sc              the type of server connection.
         * @param         isValidating    a boolean value. It is true if xml needs to
         *                                be validated, otherwise it is false.
         * @exception     DTSException    error
         *
         * @since         DTS 2.4
         */
	DbInfoServerImpl(ServerConnection sc, boolean isValidating) throws DTSException {
          	srvr = sc;
                initializeDtds(org.nci.dts.dtd.DTD.QUERY,
                               org.nci.dts.dtd.DTD.class,
                               org.nci.dts.dtd.DTD.RESULT,
                               org.nci.dts.dtd.DTD.RESULT_FILE,
                               isValidating);
	}

        /**
         * Gets the server connection type.
         *
         * @return        the type of the server connection.
         *
         * @since         DTS 2.4
         */
	protected ServerConnection getServerConnection() {
		return srvr;
	}
}