package org.nci.dts.client;

import com.apelon.dts.client.DTSException;
import org.nci.dts.EditActionConstants;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.apache.log4j.Category;
import java.sql.Date;
/**
 * <p>An EditAction contains the concept's edit action and the edit date.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Apelon Inc.</p>
 * @author Apelon Inc.
 * @version DTS 3.0
 */

public class EditAction
{
	private static Category cat = Category.getInstance(EditAction.class.getName());
        private Date editdate;
	private int action;

	/**
         * Contructs an EditAction which has an editdate and an action number.
         *
         * @param         editdate        date of the EditAction
         * @param         action          action number of the EditAction
         *
         * @since         DTS 2.4
         */
         protected EditAction(Date editdate, int action)
	{
		this.editdate = editdate;
		this.action = action;
	}

        /**
         * Gets the editdate for this EditAction.
         *
         * @return        editdate for this EditAction.
         *
         * @since         DTS 2.4
         */
	public Date getEditdate()
	{
		return editdate;
	}

        /**
         * Gets the action for this EditAction.
         *
         * @return        action for this EditAction.
         *
         * @since         DTS 2.4
         */
	public int getAction()
	{
		return action;
	}

        /**
        * Creates an EditAction by passing an XML Document.
        *
        * @param       doc       the XML document of the EditAction.
        * @return      the created EditAction.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        */
        static EditAction createEditAction(Document doc) throws DTSException
        {
                Element elem = doc.getDocumentElement();
                cat.debug("EDITACTION: " + elem.getNodeValue());
                return createEditAction(elem);
        }

        /**
        * Creates an EditAction by passing an XML Element. The Element contains
        * the information(s) about the EditAction.
        *
        * @param       elem       the XML Element of the EditAction.
        * @return      the created EditAction.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        */
        static EditAction createEditAction(Element elem) throws DTSException
        {
                String tag_name = elem.getTagName().toString().toUpperCase();
                if (tag_name.equals("NULL"))
                        return null;
                if (tag_name.equals("EDITACTION"))
                {
                        String s1 = elem.getChildNodes().item(0).getChildNodes().item(0).getNodeValue();
                        String s2 = elem.getChildNodes().item(1).getChildNodes().item(0).getNodeValue();
                        Date date = java.sql.Date.valueOf(s1);
                        int action = EditActionConstants.actionToConstant(s2);
                        EditAction editaction = new EditAction(date, action);
                        return editaction;
                }
                else
                {
                        throw new DTSException("Expected a EDITACTION tag");
                }
        }

        /**
        * Creates EditActions by passing an XML Document.
        *
        * @param       doc       the XML document of EditActions.
        * @return      an array of the created EditActions.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        */
        static EditAction[] createEditActions(Document doc) throws DTSException
        {
                Element elem = doc.getDocumentElement();
                cat.debug("EDITACTIONS: " + elem.getNodeValue());
                return createEditActions(elem);
        }

        /**
        * Creates EditActions by passing an XML Element. The Element contains
        * informations about EditActions.
        *
        * @param       elem       the XML Element of EditActions.
        * @return      an array of the created EditActions.
        * @exception   DTSException error
        *
        * @since       DTS 2.4
        */
        static EditAction[] createEditActions(Element elem) throws DTSException
        {
                String tag_name = elem.getTagName().toString().toUpperCase();
                if(tag_name.equals("NULL")) {
                        return new EditAction[0];
                }
	        else if (tag_name.equals("EDITACTIONS"))
                {
                        int n = elem.getChildNodes().getLength();
                        EditAction[] editactions = new EditAction[n];
                        int i;
                        for (i=0; i<n; i++) {
	                        editactions[i] = createEditAction((Element)elem.getChildNodes().item(i));
                        }
                        return editactions;
                }
                else
                {
                        throw new DTSException("Expected a EDITACTIONS tag");
                }
        }
}
