package org.nci.dts.db;

import com.apelon.common.sql.SQL;
import com.apelon.common.sql.ConnectionParams;
import com.apelon.common.log4j.Categories;
import com.apelon.common.log4j.LogConfigLoader;
import java.util.StringTokenizer;
import java.sql.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
/**
 * <p>Database Information Importer class. Also creates a db_info table if needed
 * <br>This class allows users to import database information into the db_info table</p>
 * <p>Copyright: Apelon Copyright (c) 2002</p>
 * <p>Company: Apelon, Inc.</p>
 * @author Apelon Inc.
 * @version 1.0
 */

public class DbInfoImporter {
        public static String USER_MANAGER_CLASS = "org.nci.dts.db.DbInfoImporter";
        public static int CREATE_DBINFO_TABLE_OPTION = 1;
        public static int ADD_DBINFO_OPTION = 2;
        public static int DELETE_DBINFO_OPTION = 3;
        public static int VIEW_OPTION = 4;
        public static int EXIT_OPTION = 5;
        public static String TABLE_NAME = "DB_INFO";
        public static String PROMPT = ">> ";

        /**
         * <p>Constructs a DbInfoImporter.
         * Makes the database connection and let user select the options.</p>
         */
        public DbInfoImporter() {
                try {
                        init();
                } catch( Exception ex ) {
                        Categories.app().error("Caught Exception",ex);
                }
        }

        /**
         * <p>Constructs a DbInfoImporter.
         * Decides if makes database connection and let user select the options according to
         * the boolean parameter</p>
         * @param init  True - makes the database connection and let user select the options.
         *              False - doesn't do anything
         */
        public DbInfoImporter(boolean init)
        {
                try {
                        if( init )
                                init();
                } catch( Exception ex ) {
                        Categories.app().error("Caught Exception",ex);
                }
        }

        public static void main(String [] args)
        {
                LogConfigLoader logConfig = new LogConfigLoader("DbInfoImporterLog.xml", DbInfoImporter.class);
                logConfig.loadDefault();

                invoke( USER_MANAGER_CLASS );
        }

        static void invoke(String classname)
        {
                try {
                        Class c = Class.forName( classname );
                        Object classObj = c.newInstance();
                } catch( InstantiationException ie ) {
                        Categories.app().error("Caught InstantiationException",ie);
                } catch( IllegalAccessException iae ) {
                        Categories.app().error("Caught IllegalAccessException", iae);
                } catch( Exception ex ) {
                        Categories.app().error("Caught Exception", ex);
                }
        }

        private void init() throws Exception
        {
                try {
                        String[] uph = SQL.promptFullConnectionArgsMultiPlatform();
                        ConnectionParams pParams = new ConnectionParams();
                        pParams.setUserName(uph[0]);
                        pParams.setPassword(uph[1]);
                        pParams.setHostName(uph[2]);
                        pParams.setPort(uph[3]);
                        pParams.setInstance(uph[4]);
                        pParams.setType(uph[5]);
                        pParams.setJDBCDriver(uph[6]);
                        pParams.setUrlTemplate(uph[7]);

                        con = SQL.getConnection(pParams);

                        if( con != null )
                                //System.out.println( "You are connected to " + uph[2] + ":" + uph[3] + ":" + uph[4] );
                                Categories.app().info("You are connected to " + uph[2] + ":" + uph[3] + ":" + uph[4]);
                        else {
                                //System.out.println( "Could not connect to " + uph[2] + ":" + uph[3] + ":" + uph[4] );
                                Categories.app().info("Could not connect to " + uph[2] + ":" + uph[3] + ":" + uph[4]);
                                System.exit(1);
                        }

                        getOptions();

                } catch(Exception ex) {
                        //System.out.println( "Exception " + ex.getMessage() );
                        Categories.app().error("Exception ", ex);
                        System.out.println( "Press Enter to continue--" );
                        System.in.read();
                        if( con == null )
                                System.exit( 1 );
                }
        }

        private void getConnection(String user, String pass, String host, String port, String inst )
            throws SQLException
        {
                int dport = new Integer(port).intValue();
                con = SQL.getConnection(user,pass,host,dport,inst);
        }

        //[1]-Add DbInfo [2]-Create DB_INFO Table [3]-Delete DbInfo [4]-Exit
        private void getOptions() throws Exception
        {
                System.out.println( "" );
                System.out.println( "Do you want to - ");
                System.out.println( "[" + CREATE_DBINFO_TABLE_OPTION + "] Create db_info table, " );
                System.out.println( "[" + ADD_DBINFO_OPTION + "] Add DbInfo, " );
                System.out.println("[" + DELETE_DBINFO_OPTION + "] Delete DbInfo, ");
                System.out.println( "[" + VIEW_OPTION + "] View all DbInfo, " );
                System.out.println( "[" + EXIT_OPTION + "] Exit" );
                System.out.print( PROMPT );
                BufferedReader in = new BufferedReader(new InputStreamReader( System.in ) );
                String option = in.readLine();
                try {
                        int value = new Integer( option ).intValue();
                        if( ( value >= CREATE_DBINFO_TABLE_OPTION ) && ( value <= EXIT_OPTION ) )
                        {
                                System.out.println( "" );
                                if( value == EXIT_OPTION ) {
                                        if( con != null )
                                                con.close();
                                        //System.out.println( "Exiting...");
                                        Categories.app().info("Existing...");
                                        System.exit(0);
                                }
                                else if( value == ADD_DBINFO_OPTION ) {
                                        promptAddDbInfo();
                                }
                                else if( value == CREATE_DBINFO_TABLE_OPTION ) {
                                        createDbInfoTable();
                                }
                                else if ( value == DELETE_DBINFO_OPTION ) {
                                        promptDeleteDbInfo();
                                }
                                else if ( value == VIEW_OPTION ) {
                                        viewDbInfo();
                                }
                        }
                        else
                                System.out.println( "Enter a valid number (" + CREATE_DBINFO_TABLE_OPTION + "-" + EXIT_OPTION + ")" );
                }
                catch( Exception ex ){
                        if( !ex.getMessage().trim().equals(""))
                                //System.out.println( "Exception : " + ex.getMessage() );
                                Categories.app().error("Exception: ", ex);
                        System.out.println( "Enter a valid number (" +
                                ADD_DBINFO_OPTION + "-" + EXIT_OPTION + ")" );
                }
                finally{
                        if( res != null )
                                res.close();
                        if( stmt != null )
                                stmt.close();
                        getOptions();
                }
        }

        /**
         * <p>Creates the db_info table. This table has three columns: db_name, version, and
         * release_date.</p>
         * @throws  SQLException
         */
        public void createDbInfoTable() throws SQLException
        {
                Statement stmt = con.createStatement();
                boolean tableExists = SQL.checkTableExists(con,TABLE_NAME);
                if( tableExists )
                        //System.out.println( "ERROR: Table '" + TABLE_NAME + "' exists!." );
                        Categories.app().error("ERROR: Table '" + TABLE_NAME + "' exists!.");
                else
                {
                        String nameField = "VARCHAR(40)";
                        String versionField = "VARCHAR(8)";
                        String dateField = "DATE";

                        int result = stmt.executeUpdate( "CREATE TABLE " + TABLE_NAME +
                                    "(\"DB_NAME\" " + nameField + " NOT NULL, \"VERSION\" " + versionField +
                                    " NOT NULL, \"RELEASE_DATE\" " + dateField + " NOT NULL)" );

                        //System.out.println( "SUCCESS: TABLE '" + TABLE_NAME + "' created for users" );
                        Categories.app().info( "SUCCESS: TABLE '" + TABLE_NAME + "' created for users" );
                }
                stmt.close();
        }

        private void promptAddDbInfo() throws Exception
        {
                if( con != null ){
                        boolean tableExists = SQL.checkTableExists(con,TABLE_NAME);
                        if (!tableExists) {
                                //System.out.println("DB_INFO table doesn't exist. You have to create the table at first.");
                                Categories.app().error("DB_INFO table doesn't exist. You have to create the table at first.");
                                return;
                        }
                        boolean promptAgain = true;
                        BufferedReader in = new BufferedReader(new InputStreamReader( System.in ) );
                        String dbname = null;
                        String dbversion = null;
                        while(promptAgain) {
                                System.out.println( "Enter the name of the database - " );
                                System.out.print( PROMPT );
                                dbname = in.readLine();
                                if( dbname == null || dbname.equals("") || dbname.equals("\n\r") )
                                        promptAgain = true;
                                else
                                        promptAgain = false;
                        }

                        promptAgain = true;
                        while(promptAgain) {
                                System.out.println( "Enter the version of the database - " );
                                System.out.print( PROMPT );
                                dbversion = in.readLine();
                                if( dbversion == null || dbversion.equals("") || dbversion.equals("\n\r") )
                                        promptAgain = true;
                                else
                                        promptAgain = false;
                        }

                        promptAgain = true;
                        String dbdate = null;
                        while(promptAgain) {
                                System.out.println( "Enter the product date for the database ");
                                System.out.println( "Please follow the format: DD-MON-YYYY (e.g. 02-Jun-1999) ");
                                System.out.println( "   - DD is the day of the month (1 through 31), as one or two decimal digits. ");
                                System.out.println( "   - MON is the month in (Jan, Feb, Mar, Apr, May, Jun, Jul, Aug, Sep, Oct, Nov, Dec). ");
                                System.out.println( "   - YYYY is the year, as four decimal digits. ");
                                System.out.print( PROMPT );
                                dbdate = in.readLine();
                                int day = -1;
                                String month = null;
                                int year = -1;
                                if( dbdate == null || dbdate.equals("") || dbdate.equals("\n\r") ) {
                                        promptAgain = true;
                                } else {
                                        StringTokenizer st=new StringTokenizer(dbdate, "-");
                                        if (st.countTokens()==3) {
                                                day = Integer.parseInt((String)st.nextElement());
                                                month = (String)st.nextElement();
                                                year = Integer.parseInt((String)st.nextElement());
                                                boolean invalidInput = false;
                                                if (day<1 || day>31) {
                                                        //System.out.println("Invalid day!");
                                                        Categories.app().error("Invalid day!");
                                                        invalidInput = true;
                                                }
                                                if (month.equalsIgnoreCase("Jan") || month.equalsIgnoreCase("January")
                                                || month.equalsIgnoreCase("Feb") || month.equalsIgnoreCase("February")
                                                || month.equalsIgnoreCase("Mar") || month.equalsIgnoreCase("March")
                                                || month.equalsIgnoreCase("Apr") || month.equalsIgnoreCase("April")
                                                || month.equalsIgnoreCase("May")
                                                || month.equalsIgnoreCase("Jun") || month.equalsIgnoreCase("June")
                                                || month.equalsIgnoreCase("Jul") || month.equalsIgnoreCase("July")
                                                || month.equalsIgnoreCase("Aug") || month.equalsIgnoreCase("August")
                                                || month.equalsIgnoreCase("Sep") || month.equalsIgnoreCase("September")
                                                || month.equalsIgnoreCase("Oct") || month.equalsIgnoreCase("October")
                                                || month.equalsIgnoreCase("Nov") || month.equalsIgnoreCase("November")
                                                || month.equalsIgnoreCase("Dec") || month.equalsIgnoreCase("December")) {
                                                } else {
                                                        //System.out.println("Invalid month!");
                                                        Categories.app().error("Invalid month!");
                                                        invalidInput = true;
                                                }
                                                if (year<1000 || year>9999) {
                                                        //System.out.println("Invalid year!");
                                                        Categories.app().error("Invalid year!");
                                                        invalidInput = true;
                                                }
                                                if (invalidInput) {
                                                        promptAgain = true;
                                                } else {
                                                        promptAgain = false;
                                                }
                                        } else {
                                                //System.out.println("Invalid Input! Please follow the required format.");
                                                Categories.app().error("Invalid Input! Please follow the required format.");
                                                promptAgain = true;
                                        }
                                }
                        }
                        try {
                                //File file = new File(filename);
                                importDbInfo(dbname, dbversion, dbdate);
                        } catch( Exception ex) {
                                //System.out.println( "Error: " + ex.getMessage() );
                                Categories.app().error("Error: ", ex);
                        }
                }
        }

        private void promptDeleteDbInfo() throws Exception
        {
                if( con != null ){
                        boolean tableExists = SQL.checkTableExists(con,TABLE_NAME);
                        if (!tableExists) {
                                //System.out.println("DB_INFO table doesn't exist. You have to create the table at first.");
                                Categories.app().error("DB_INFO table doesn't exist. You have to create the table at first.");
                                return;
                        }
                        boolean promptAgain = true;
                        BufferedReader in = new BufferedReader(new InputStreamReader( System.in ) );
                        String dbname = null;
                        String dbversion = null;
                        while(promptAgain) {
                                System.out.println( "Enter the name of the database - " );
                                System.out.print( PROMPT );
                                dbname = in.readLine();
                                if( dbname == null || dbname.equals("") || dbname.equals("\n\r") )
                                        promptAgain = true;
                                else
                                        promptAgain = false;
                        }

                        promptAgain = true;
                        while(promptAgain) {
                                System.out.println( "Enter the version of the database - " );
                                System.out.print( PROMPT );
                                dbversion = in.readLine();
                                if( dbversion == null || dbversion.equals("") || dbversion.equals("\n\r") )
                                        promptAgain = true;
                                else
                                        promptAgain = false;
                        }
                        try {
                                //File file = new File(filename);
                                deleteDbInfo(dbname, dbversion);
                        } catch( Exception ex) {
                                //System.out.println( "Error: " + ex.getMessage() );
                                Categories.app().error("Error: ", ex);
                        }
                }
        }

        /**
         * <p>Imports database information into db_info table.</p>
         * @param   name      database name
         * @param   version   database version
         * @param   date      database release date
         * @throws  SQLException
         */
        public void importDbInfo(String name, String version, String date) throws SQLException
        {
                stmt = con.prepareStatement("INSERT INTO DB_INFO VALUES (?,?,?)");
                stmt.setString(1,name);
                stmt.setString(2,version);
                stmt.setString(3,date);
                int result = stmt.executeUpdate();
                if( result == 1 )
                        //System.out.println( "SUCCESS: Db_Info added to the database" );
                        Categories.app().info( "SUCCESS: Db_Info added to the database" );
                else
                        //System.out.println( "UNSUCCESS: Db_Info NOT added to the database" );
                        Categories.app().error( "UNSUCCESS: Db_Info NOT added to the database" );
        }

        /**
         * <p>Deletes database information from db_info table.</p>
         * @param   name      database name
         * @param   version   database version
         * @throws  SQLException
         */
        public void deleteDbInfo(String name, String version) throws SQLException
        {
                if( con != null ){
                        stmt = con.prepareStatement("DELETE FROM " + TABLE_NAME + " WHERE db_name = ? AND version = ? " );
                        stmt.setString( 1, name );
                        stmt.setString(2, version);
                        int result = stmt.executeUpdate();

                        if( result == 1 )
                                //System.out.println( "SUCCESS: Db_Info '" + name + "' deleted from the database" );
                                Categories.app().info( "SUCCESS: Db_Info '" + name + "' deleted from the database" );
                        else
                                //System.out.println( "ERROR: Db_Info '" + name + "' not found in database!" );
                                Categories.app().error( "ERROR: Db_Info '" + name + "' not found in database!" );
                }
        }

        /**
         * <p>Views database information in db_info table.</p>
         * @throws  SQLException
         */
        public void viewDbInfo() throws SQLException
        {
                System.out.println( "****************** List of Database Information ****************** " );
                Statement stmt = con.createStatement();
                res = stmt.executeQuery("SELECT * FROM " + TABLE_NAME );
                int count = 0;
                String name = "";
                String version = "";
                Date date = null;
                while( res.next() ) {
                        name = res.getString(1);
                        version = res.getString(2);
                        date = res.getDate(3);
                        System.out.println( "(" + ++count + ") name-" + name + "\t version-" + version + "\t release date-" + date.toString() );
                }
                if( count == 0 )
                        System.out.println( "  No DbInfo in database" );

                System.out.println( "****************************************************************** " );
                res.close();
                stmt.close();
        }

        private Connection con = null;
        private PreparedStatement stmt = null;
        private ResultSet res = null;
}