package org.nci.dts.samples;

import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import com.apelon.common.log4j.Categories;
import com.apelon.common.log4j.LogConfigLoader;
import com.apelon.apelonserver.client.ServerConnection;
import com.apelon.apelonserver.client.ServerConnectionSocket;
import com.apelon.apelonserver.client.ApelonException;
import com.apelon.dts.client.DTSException;

/**
 * <p>The SampleTest is the main class to run the sample test.
 * <br>In this class the users will decide if they want to test methods in ConceptHistoryServer,
 * and/or test methods in DbInfoServer.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Apelon Inc.</p>
 * @author Apelon Inc.
 * @version 1.0
 */

public class SampleTest {
  private boolean prompt = false;
  private String host;
  private int port;
  private ServerConnection srvr_conn;
  private String tch;
  private String tdb;
  private BufferedReader in;

  /**
   * Contructs a SampleTest which invokes GetTestsToBeRun method.
   *
   * @since         DTS 2.4
   */
  public SampleTest(boolean prompt) {

    if (prompt) {
      this.prompt = true;
    }

    try {
      GetTestsToBeRun();
    }
    catch (DTSException dtsex) {
      Categories.dataClient().error("Caught DTSException", dtsex);
    }
    catch (IOException io) {
      Categories.dataClient().error("Caught IOException", io);
    }
  }

  public boolean getPrompt(){
    return prompt;
  }

  /**
   * This method does two things.
       * <br>1. Lets user determine which test will be run. There are two tests class.
       * One class tests all methods in the org.nci.dts.client.ConceptHistoryServer.
   * The other class tests all methods in the org.nci.dts.client.DbInfoServer.
   * <br>2. Set up Socket Server Connection.
   *
   * @since         DTS 2.4
   */
  public void GetTestsToBeRun() throws IOException, DTSException {
    if (!prompt) {
      tch = "yes";
      tdb = "yes";
      host = "wine";
      port = 6666;
    }
    else {
      BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
      boolean validin = false;

      while (!validin) {
        System.out.println("Please specify the test you'd like to run, ");
        System.out.print(
            "1. Test the ConceptHistoryServer? (answer 'yes' or 'no') ");
        tch = in.readLine();
        if (tch.equalsIgnoreCase("yes") || tch.equalsIgnoreCase("no")) {
          validin = true;
        }
        else {
          continue;
        }
        System.out.print("2. Test the DbInfoServer? (answer 'yes' or 'no') ");
        tdb = in.readLine();
        if (tdb.equalsIgnoreCase("yes") || tdb.equalsIgnoreCase("no")) {
          validin = true;
        }
        else {
          validin = false;
        }
      }
      if (tch.equalsIgnoreCase("yes") || tdb.equalsIgnoreCase("yes")) {
        System.out.print("Host: ");
        host = in.readLine();
        System.out.print("Port: ");
        port = new Integer(in.readLine()).intValue();
      }
    }
    try {
      srvr_conn = new ServerConnectionSocket(host, port);
    }
    catch (ApelonException ex) {
      throw new DTSException("Error making socket connection.", ex);
    }
  }

  /**
   * This is the main method.
   *
   * @since         DTS 2.4
   */
  public static void main(String[] args) {
    LogConfigLoader logConfig = new LogConfigLoader("NCISampleTestLog.xml",
        SampleTest.class);
    logConfig.loadDefault();

    SampleTest test = new SampleTest(args.length == 0);
    if (test.tch.equalsIgnoreCase("yes")) {
      Categories.dataClient().info("Start ConceptHistoryServer test...");
      TestConHis tconhis = new TestConHis(test.getPrompt());
      tconhis.initServers(test.srvr_conn);
      tconhis.run();
      Categories.dataClient().info(
          "The ConceptHistoryServer test is finished.\n");
    }
    if (test.tdb.equalsIgnoreCase("yes")) {
      Categories.dataClient().info("Start DbInfoServer test...");
      TestDbInfo tdbinfo = new TestDbInfo(test.getPrompt());
      tdbinfo.initServers(test.srvr_conn);
      tdbinfo.run();
      Categories.dataClient().info("The DbInfoServer test is finished.\n");
    }

    try {
      System.out.println("Hit the ENTER to terminate.");
      System.in.read();
      System.exit(0);
    }
    catch (Exception ee) {
      Categories.dataClient().error("Caught Exception", ee);
    }
  }
}