package org.nci.dts.samples;

import java.io.*;
import java.io.PrintWriter;
import java.sql.Date;
import java.util.Vector;
import com.apelon.apelonserver.client.*;
import com.apelon.apelonserver.server.ApelonServer;
import com.apelon.common.xml.*;
import com.apelon.common.log4j.Categories;
import com.apelon.common.log4j.LogConfigLoader;
import org.nci.dts.client.*;
import org.nci.dts.EditActionConstants;
/**
 * <p>This class tests all methods in org.nci.dts.client.ConceptHistoryServer.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Apelon Inc.</p>
 * @author Apelon Inc.
 * @version 1.0
 */

public class TestConHis {
  private boolean prompt = false;
        private ConceptHistoryServer chsr;
        private BufferedReader in;
        private Date startDate;
        private Date endDate;
        private Date baselineDate;
        private String conceptCode;
        private int namespaceId;

        /**
         * Constructs a TestConHis.
         *
         * @since         DTS 2.4
         */
        public TestConHis(boolean prompt){
          this.prompt = prompt;
        }

        /**
         * Creates a ConceptHistoryServer instance and lets user input all required information.
         *
         * @param         srvr_conn         the ServerConnection.
         *
         * @since         DTS 2.4
         */
        public void initServers(ServerConnection srvr_conn)
        {
                try {
                  if (prompt) {
                        in = new BufferedReader(new InputStreamReader(System.in));
                        chsr = ConceptHistoryServer.createInstance(srvr_conn);

                        System.out.println("Please enter the following information. When you enter a Date, please follow the format like 1999-01-26.");
                        System.out.print("Start Date: ");
                        startDate = Date.valueOf(in.readLine());
                        System.out.print("End Date: ");
                        endDate = Date.valueOf(in.readLine());
                        System.out.print("Baseline Date: ");
                        baselineDate = Date.valueOf(in.readLine());
                        System.out.print("Namespace Id: ");
                        namespaceId = Integer.parseInt(in.readLine());
                        System.out.print("Concept Code: ");
                        conceptCode = in.readLine();
                  }
                  else {
                        startDate = Date.valueOf("1999-01-01");
                        endDate = Date.valueOf("1999-01-03");
                        baselineDate = Date.valueOf("1999-01-01");
                        namespaceId = 1079;
                        conceptCode = "C0001";
                  }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the getHistoryDates method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @since         DTS 2.4
         */
        public void getHistoryDates()
        {
                try{
                        if( chsr != null) {
                                Date[] dates = chsr.getHistoryDates();
                                if (dates == null) {
                                        Categories.dataClient().info("No matching result.");
                                } else {
                                        for (int i=0; i < dates.length; i++) {
                                                Categories.dataClient().info("Dates in History table: " + dates[i].toString());
                                        }
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }

        }

        /**
         * Tests the getHistoryStart method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @since         DTS 2.4
         */
        public void getHistoryStart()
        {
                try{
                        if( chsr != null) {
                                Date date = chsr.getHistoryStart();
                                if (date == null) {
                                        Categories.dataClient().info("No matching result.");
                                } else {
                                        Categories.dataClient().info("Start Date in History table: " + date.toString());
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the getHistoryEnd method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @since         DTS 2.4
         */
        public void getHistoryEnd()
        {
                try{
                        if( chsr != null) {
                                Date date = chsr.getHistoryEnd();
                                if (date == null) {
                                        Categories.dataClient().info("No matching result.");
                                } else {
                                        Categories.dataClient().info("End Date in History table: " + date.toString());
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the getConceptCreationDate method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @param         inCode          The specified concept's code.
         *
         * @since         DTS 2.4
         */
        public void getConceptCreationDate(int namespaceId, String inCode)
        {
                try{
                        if( chsr != null) {
                                Date date = chsr.getConceptCreationDate(namespaceId, inCode);
                                if (date == null) {
                                        Categories.dataClient().info("No matching result.");
                                } else {
                                        Categories.dataClient().info("Creation Date of " + inCode + " : " + date.toString());
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the getDescendantCodes method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @param         inCode             The specified concept's code.
         * @param         atBaseline         The boolean value to specify the search type.
         * @param         iBaselineDate      The initial baseline date.
         * @param         fBaselineDate      The final baseline date.
         *
         * @since         DTS 2.4
         */
        public void getDescendantCodes(int namespaceId, String inCode, boolean atBaseline, Date iBaselineDate, Date fBaselineDate)
        {
                try{
                        if( chsr != null) {
                                String[] codes = chsr.getDescendantCodes(namespaceId, inCode, atBaseline, iBaselineDate, fBaselineDate);
                                if (codes == null) {
                                        Categories.dataClient().info("No matching result.");
                                } else {
                                        for (int i=0; i<codes.length; i++) {
                                                Categories.dataClient().info("Descendant codes of " + inCode + " (" + iBaselineDate.toString() + " To "
                                                + fBaselineDate.toString() + "): " + codes[i]);
                                        }
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the getAncestorCodes method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @param         inCode             The specified concept's code.
         * @param         atBaseline         The boolean value to specify the search type.
         * @param         iBaselineDate      The initial baseline date.
         * @param         fBaselineDate      The final baseline date.
         *
         * @since         DTS 2.4
         */
        public void getAncestorCodes(int namespaceId, String inCode, boolean atBaseline, Date iBaselineDate, Date fBaselineDate)
        {
                try{
                        if( chsr != null) {
                                String[] codes = chsr.getAncestorCodes(namespaceId, inCode, atBaseline, iBaselineDate, fBaselineDate);
                                if (codes == null) {
                                        Categories.dataClient().info("No matching result.");
                                } else {
                                        for (int i=0; i<codes.length; i++) {
                                                Categories.dataClient().info("Ancestor codes of " + inCode + " (" + iBaselineDate.toString() + " To "
                                                + fBaselineDate.toString() + "): " + codes[i]);
                                        }
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the getCodeActionChildren method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @param         inCode             The specified concept's code.
         * @param         baselineDate       The specified baseline date.
         *
         * @since         DTS 2.4
         */
        public void getCodeActionChildren(int namespaceId, String inCode, Date baselineDate)
        {
                try{
                        if( chsr != null) {
                                String[] codes = chsr.getCodeActionChildren(namespaceId, inCode, baselineDate);
                                if (codes == null) {
                                        Categories.dataClient().info("No matching result.");
                                } else {
                                        for (int i=0; i<codes.length; i++) {
                                                Categories.dataClient().info("Children codes of " + inCode + " (" + baselineDate.toString() + "): " + codes[i]);
                                        }
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the getCodeActionParents method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @param         inCode             The specified concept's code.
         * @param         baselineDate       The specified baseline date.
         *
         * @since         DTS 2.4
         */
        public void getCodeActionParents(int namespaceId, String inCode, Date baselineDate)
        {
                try{
                        if( chsr != null) {
                                String[] codes = chsr.getCodeActionParents(namespaceId, inCode, baselineDate);
                                if (codes == null) {
                                        Categories.dataClient().info("No matching result.");
                                } else {
                                        for (int i=0; i<codes.length; i++) {
                                                Categories.dataClient().info("Parents codes of " + inCode + " (" + baselineDate.toString() + "): " + codes[i]);
                                        }
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the getDescendantCodeActionDates method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @param         inCode             The specified concept's code.
         * @param         iBaselineDate      The initial baseline date.
         * @param         fBaselineDate      The final baseline date.
         *
         * @since         DTS 2.4
         */
        public void getDescendantCodeActionDates(int namespaceId, String inCode, Date iBaselineDate, Date fBaselineDate)
        {
                try{
                        if( chsr != null) {
                                CodeActionDate[] cads = chsr.getDescendantCodeActionDates(namespaceId, inCode, iBaselineDate, fBaselineDate);
                                Categories.dataClient().info("Descendant CodeActionDates of " + inCode + " (" + iBaselineDate.toString()
                                    + " To " + fBaselineDate.toString() + "): ");
                                if (cads == null) {
                                        Categories.dataClient().info("No matching result.");
                                } else {
                                        for (int i=0; i<cads.length; i++) {
                                                Categories.dataClient().info("Code - " + cads[i].getCode() + "\t Date - " + cads[i].getDate().toString());
                                        }
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the getAncestorCodeActionDates method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @param         inCode             The specified concept's code.
         * @param         iBaselineDate      The initial baseline date.
         * @param         fBaselineDate      The final baseline date.
         *
         * @since         DTS 2.4
         */
        public void getAncestorCodeActionDates(int namespaceId, String inCode, Date iBaselineDate, Date fBaselineDate)
        {
                try{
                        if( chsr != null) {
                                CodeActionDate[] cads = chsr.getAncestorCodeActionDates(namespaceId, inCode, iBaselineDate, fBaselineDate);
                                Categories.dataClient().info("Ancestor CodeActionDates of " + inCode + " (" + iBaselineDate.toString()
                                    + " To " + fBaselineDate.toString() + "): ");
                                if (cads == null) {
                                        Categories.dataClient().info("No matching result.");
                                } else {
                                        for (int i=0; i<cads.length; i++) {
                                                Categories.dataClient().info("Code - " + cads[i].getCode() + "\t Date - " + cads[i].getDate().toString());
                                        }
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the getConceptEditActionDates method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @param         inCode             The specified concept's code.
         * @param         iAction            The specified editing action.
         *
         * @since         DTS 2.4
         */
        public void getConceptEditActionDates(int namespaceId, String inCode, int iAction)
        {
                try{
                        if( chsr != null) {
                                Date[] dates = chsr.getConceptEditActionDates(namespaceId, inCode, iAction);
                                if (dates == null) {
                                        Categories.dataClient().info("No matching result.");
                                } else {
                                        for (int i=0; i<dates.length; i++) {
                                                Categories.dataClient().info("Dates of " + inCode + " doing " + iAction + ": " + dates[i].toString());
                                        }
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the getConceptEditAction method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @param         inCode             The specified concept's code.
         *
         * @since         DTS 2.4
         */
        public void getConceptEditAction(int namespaceId, String inCode)
        {
                try{
                        if( chsr != null) {
                                EditAction[] eas = chsr.getConceptEditAction(namespaceId, inCode);
                                Categories.dataClient().info("EditActions of " + inCode + ": ");
                                if (eas == null) {
                                        Categories.dataClient().info("No matching result.");
                                } else {
                                        for (int i=0; i<eas.length; i++) {
                                                Categories.dataClient().info("Date---" + eas[i].getEditdate().toString() + "\t Action---"
                                                + EditActionConstants.actionToString(eas[i].getAction()));
                                        }
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the isConceptRetired method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @param         inCode             The specified concept's code.
         *
         * @since         DTS 2.4
         */
        public void isConceptRetired(int namespaceId, String inCode)
        {
                try{
                        if( chsr != null) {
                                boolean isRetired = chsr.isConceptRetired(namespaceId, inCode);
                                if (isRetired) {
                                        Categories.dataClient().info(inCode + " is retired.");
                                } else {
                                        Categories.dataClient().info(inCode + " isn't retired.");
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the isConceptEdited method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @param         inCode             The specified concept's code.
         * @param         mrg_splt           True - only check for "merge or split" editing action.
         *                                   False - check for any editing action.
         * @param         iBaselineDate      The initial baseline date.
         * @param         fBaselineDate      The final baseline date.
         *
         * @since         DTS 2.4
         */
        public void isConceptEdited(int namespaceId, String inCode, boolean mrg_splt, Date iBaselineDate, Date fBaselineDate)
        {
                try{
                        if( chsr != null) {
                                boolean isEdited = chsr.isConceptEdited(namespaceId, inCode, mrg_splt, iBaselineDate, fBaselineDate);
                                if (isEdited) {
                                        Categories.dataClient().info(inCode + " is edited.");
                                } else {
                                        Categories.dataClient().info(inCode + " isn't edited.");
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the getNumberEditActions method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @param         inCode             The specified concept's code.
         * @param         baselineDate       The specified baseline date.
         *
         * @since         DTS 2.4
         */
        public void getNumberEditActions(int namespaceId, String inCode, Date baselineDate)
        {
                try{
                        if( chsr != null) {
                                int num = chsr.getNumberEditActions(namespaceId, inCode, baselineDate);
                                Categories.dataClient().info("The edit action number of " + inCode + " in " + baselineDate.toString()
                                + " is: " + num);
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the getNumberEditActions method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @param         inCode             The specified concept's code.
         * @param         iBaselineDate      The initial baseline date.
         * @param         fBaselineDate      The final baseline date.
         *
         * @since         DTS 2.4
         */
        public void getNumberEditActions(int namespaceId, String inCode, Date iBaselineDate, Date fBaselineDate)
        {
                try{
                        if( chsr != null) {
                                int num = chsr.getNumberEditActions(namespaceId, inCode, iBaselineDate, fBaselineDate);
                                Categories.dataClient().info("The edit action number of " + inCode + " from " + iBaselineDate.toString()
                                + " to " + fBaselineDate.toString() + " is: " + num);
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the getConceptEditAction method in org.nci.dts.client.ConceptHistoryServer.
         *
         * @param         inCode             The specified concept's code.
         * @param         baselineDate       The specified baseline date.
         *
         * @since         DTS 2.4
         */
        public void getConceptEditAction(int namespaceId, String inCode, Date baselineDate)
        {
                try{
                        if( chsr != null) {
                                int[] actions = chsr.getConceptEditAction(namespaceId, inCode, baselineDate);
                                Categories.dataClient().info("The concept " + inCode + " in " + baselineDate.toString()
                                + " has the following edit action: ");
                                if (actions == null) {
                                        Categories.dataClient().info("No matching result.");
                                } else {
                                        for (int i=0; i<actions.length; i++) {
                                                Categories.dataClient().info(EditActionConstants.actionToString(actions[i]));
                                        }
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * This method invokes all test methods in this class.
         *
         * @since         DTS 2.4
         */
        public void run ()
        {
                Categories.dataClient().info("Starting to test getHistoryDates...");
                getHistoryDates();
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test getHistoryStart...");
                getHistoryStart();
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test getHistoryEnd...");
                getHistoryEnd();
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test getConceptCreationDate...");
                getConceptCreationDate(namespaceId, conceptCode);
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test getDescendantCodes...");
                Categories.dataClient().info("Boolean value is true---");
                getDescendantCodes(namespaceId, conceptCode, true, startDate, endDate);
                Categories.dataClient().info("Boolean value is false---");
                getDescendantCodes(namespaceId, conceptCode, false, startDate, endDate);
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test getAncestorCodes...");
                Categories.dataClient().info("Boolean value is true---");
                getAncestorCodes(namespaceId, conceptCode, true, startDate, endDate);
                Categories.dataClient().info("Boolean value is false---");
                getAncestorCodes(namespaceId, conceptCode, false, startDate, endDate);
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test getCodeActionChildren...");
                getCodeActionChildren(namespaceId, conceptCode, baselineDate);
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test getCodeActionParents...");
                getCodeActionParents(namespaceId, conceptCode, baselineDate);
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test getDescendantCodeActionDates...");
                getDescendantCodeActionDates(namespaceId, conceptCode, startDate, endDate);
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test getAncestorCodeActionDates...");
                getAncestorCodeActionDates(namespaceId, conceptCode, startDate, endDate);
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test getConceptEditActionDates...");
                Categories.dataClient().info("Testing the create action---");
                getConceptEditActionDates(namespaceId, conceptCode, EditActionConstants.EDIT_ACTION_CREATE);
                Categories.dataClient().info("Testing the modify action---");
                getConceptEditActionDates(namespaceId, conceptCode, EditActionConstants.EDIT_ACTION_MODIFY);
                Categories.dataClient().info("Testing the split action---");
                getConceptEditActionDates(namespaceId, conceptCode, EditActionConstants.EDIT_ACTION_SPLIT);
                Categories.dataClient().info("Testing the merge action---");
                getConceptEditActionDates(namespaceId, conceptCode, EditActionConstants.EDIT_ACTION_MERGE);
                Categories.dataClient().info("Testing the retire action---");
                getConceptEditActionDates(namespaceId, conceptCode, EditActionConstants.EDIT_ACTION_RETIRE);
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test getConceptEditAction...");
                Categories.dataClient().info("Without a baseline date ---- ");
                getConceptEditAction(namespaceId, conceptCode);
                Categories.dataClient().info("With a baseline date ---- ");
                getConceptEditAction(namespaceId, conceptCode, baselineDate);
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test isConceptRetired...");
                isConceptRetired(namespaceId, conceptCode);
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test isConceptEdited...");
                Categories.dataClient().info("Boolean value is true---");
                isConceptEdited(namespaceId, conceptCode, true, startDate, endDate);
                Categories.dataClient().info("Boolean value is false---");
                isConceptEdited(namespaceId, conceptCode, false, startDate, endDate);
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test getNumberEditActions...");
                getNumberEditActions(namespaceId, conceptCode, baselineDate);
                getNumberEditActions(namespaceId, conceptCode, startDate, endDate);
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test getConceptEditAction...");
                getConceptEditAction(namespaceId, conceptCode, baselineDate);
                Categories.dataClient().info("The test ended...");
        }
}
