package org.nci.dts.samples;

import java.io.*;
import java.io.PrintWriter;
import java.sql.Date;
import java.util.Vector;
import com.apelon.apelonserver.client.*;
import com.apelon.apelonserver.server.ApelonServer;
import com.apelon.common.xml.*;
import com.apelon.common.log4j.Categories;
import com.apelon.common.log4j.LogConfigLoader;
import org.nci.dts.client.*;
import org.nci.dts.EditActionConstants;
/**
 * <p>This class tests all methods in org.nci.dts.client.DbInfoServer.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Apelon Inc.</p>
 * @author Apelon Inc.
 * @version 1.0
 */

public class TestDbInfo {
  private boolean prompt = false;
        private DbInfoServer disr;
        private BufferedReader in;
        private String dbname;

        /**
         * Constructs a TestDbInfo.
         *
         * @since         DTS 2.4
         */
        public TestDbInfo(boolean prompt){
          this.prompt = prompt;
        }

        /**
         * Creates a DbInfoServer instance and lets user input all required information.
         *
         * @param         srvr_conn         the ServerConnection.
         *
         * @since         DTS 2.4
         */
        public void initServers(ServerConnection srvr_conn)
        {
          if (prompt) {

                try {
                        in = new BufferedReader(new InputStreamReader(System.in));
                        disr = DbInfoServer.createInstance(srvr_conn);

                        System.out.println("Please enter the following information.");
                        System.out.print("Database Name: ");
                        dbname = in.readLine();
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
          }
          else {
            dbname="dts3nci";
          }
        }

        /**
         * Tests the getDbInfo method in org.nci.dts.client.DbInfoServer.
         *
         * @param          dbname    - the database name.
         *
         * @since         DTS 2.4
         */
        public void getDbInfo(String name)
        {
                try{
                        if( disr != null) {
                                DbInfo dbinfo = disr.getDbInfo(name);
                                if (dbinfo == null) {
                                        Categories.dataClient().info("No matching result.");
                                } else {
                                        Categories.dataClient().info("The database information is like: ");
                                        Categories.dataClient().info("db name - " + dbinfo.getName());
                                        Categories.dataClient().info("version - " + dbinfo.getVersion());
                                        Categories.dataClient().info("release date - " + dbinfo.getReleaseDate());
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * Tests the getDbInfos method in org.nci.dts.client.DbInfoServer.
         *
         * @since         DTS 2.4
         */
        public void getDbInfos()
        {
                try{
                        if( disr != null) {
                                DbInfo[] dbinfos = disr.getDbInfos();
                                Categories.dataClient().info("The database information is like: ");
                                if (dbinfos == null) {
                                        Categories.dataClient().info("No matching result.");
                                } else {
                                        for (int i=0; i<dbinfos.length; i++) {
                                                Categories.dataClient().info("#" + (i+1) + " db name - " + dbinfos[i].getName());
                                                Categories.dataClient().info("#" + (i+1) + " version - " + dbinfos[i].getVersion());
                                                Categories.dataClient().info("#" + (i+1) + " release date - " + dbinfos[i].getReleaseDate());
                                        }
                                }
                        }
                } catch (Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         * This method invokes all test methods in this class.
         *
         * @since         DTS 2.4
         */
        public void run ()
        {
                Categories.dataClient().info("Starting to test getDbInfo...");
                getDbInfo(dbname);
                Categories.dataClient().info("The test ended...\n");

                Categories.dataClient().info("Starting to test getDbInfos...");
                getDbInfos();
                Categories.dataClient().info("The test ended...");
        }
}
