package org.nci.dts.server;

import org.nci.dts.db.ConceptHistoryDb;
import org.nci.dts.db.DbInfoDb;
import com.apelon.common.sql.SQL;
import com.apelon.common.xml.XML;
import com.apelon.common.xml.XMLException;
import com.apelon.common.version.Version;
import com.apelon.common.sql.DbConnExceptionVerifier;
import com.apelon.common.log4j.Categories;

import com.apelon.apelonserver.server.Permissions;
import com.apelon.apelonserver.ErrorMarshaller;

import java.io.*;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.DOMException;


/**
 * <p>ConceptHistoryServer is a server side class which resolves the request from the
 * ConceptHistoryServer class in client side and returns the result to client side. </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Apelon Inc.</p>
 * @author Apelon Inc.
 * @version 1.0
 */

public class QueryServer extends com.apelon.apelonserver.server.QueryServer
{
        private ConceptHistoryDb fDb;
        private DbInfoDb fDbInfo;
        protected SQLException fDbInitException;

        /**
         * Constructs a QueryServer.
         */
        public QueryServer(){}

        /**
         * This init method sets the dtd file.
         */
        public void init()
        {
                try
                {
                  setDtd(org.nci.dts.dtd.DTD.class,
                         org.nci.dts.dtd.DTD.QUERY,
                         org.nci.dts.dtd.DTD.QUERY_FILE);
                }
                catch (Exception e) { log("failed to init application object",e); }
        }

         // accessors
        public void setConnection(Connection c) {
          super.setConnection(c);
          setDb(c);
        }

        /**
         * Closes the database connection.
         * @throws Exception
         */
        public void close() throws Exception
        {
                db().close();
                dbinfo().close();
        }

        public boolean queyrServerIsValid(){
          return true;
        }

        /**
         * Checks if the database connection is open.
         * @return  a boolean.    True - the connection is open.
         *                        False - the connection is closed.
         */
        public boolean isOpen()
        {
                try
                {
                        return (!db().isClosed() && !dbinfo().isClosed());
                }
                catch (SQLException e)
                {
                        log("database connection is no longer open",e);
                        return false;
                }
        }

        /**
         * Gets the date(s) in the concept_history table.
         * <p>This method calls the corresponding method in org.nci.dts.db.ConceptHistoryDb.</p>
         * @param   root    the XML Document of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String getDate(Document doc, Permissions permit)
            throws SQLException, XMLException
        {
                return db().executeGetDate(doc.getDocumentElement());
        }

        /**
         * Gets the concept edit date(s).
         * <p>This method calls the corresponding method in org.nci.dts.db.ConceptHistoryDb.</p>
         * @param   root    the XML Document of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String getConceptDate(Document doc, Permissions permit)
            throws SQLException, XMLException
        {
                return db().executeGetConceptDate(doc.getDocumentElement());
        }

        /**
         * Gets the descendant/ancestor concept codes in the date range.
         * <p>This method calls the corresponding method in org.nci.dts.db.ConceptHistoryDb.</p>
         * @param   root    the XML Document of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String getCodesInDateRange(Document doc, Permissions permit)
            throws SQLException, XMLException
        {
                return db().executeGetCodesInDateRange(doc.getDocumentElement());
        }

        /**
         * Gets the descendant/ancestor concept codes in the baseline date.
         * <p>This method calls the corresponding method in org.nci.dts.db.ConceptHistoryDb.</p>
         * @param   root    the XML Document of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String getCodesInDate(Document doc, Permissions permit)
            throws SQLException, XMLException
        {
                return db().executeGetCodesInDate(doc.getDocumentElement());
        }

        /**
         * Gets the descendant/ancestor's concept codes and the corresponding baseline
         * dates in the date range.
         * <p>This method calls the corresponding method in org.nci.dts.db.ConceptHistoryDb.</p>
         * @param   root    the XML Document of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String getCodeActionDate(Document doc, Permissions permit)
            throws SQLException, XMLException
        {
                return db().executeGetCodeActionDate(doc.getDocumentElement());
        }

        /**
         * Gets the specified concept's edit actions and the corresponding edit dates.
         * <p>This method calls the corresponding method in org.nci.dts.db.ConceptHistoryDb.</p>
         * @param   root    the XML Document of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String getEditAction(Document doc, Permissions permit)
            throws SQLException, XMLException
        {
                return db().executeGetEditAction(doc.getDocumentElement());
        }

        /**
         * Checks if the specified concept is retired or not.
         * <p>This method calls the corresponding method in org.nci.dts.db.ConceptHistoryDb.</p>
         * @param   root    the XML Document of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String isRetired(Document doc, Permissions permit)
            throws SQLException, XMLException
        {
                return db().executeIsRetired(doc.getDocumentElement());
        }

        /**
         * Checks if the specified concept is edited or not in the date range.
         * <p>This method calls the corresponding method in org.nci.dts.db.ConceptHistoryDb.</p>
         * @param   root    the XML Document of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String isEdited(Document doc, Permissions permit)
            throws SQLException, XMLException
        {
                return db().executeIsEdited(doc.getDocumentElement());
        }

        /**
         * Gets the specified concept's edit actions in the baseline date.
         * <p>This method calls the corresponding method in org.nci.dts.db.ConceptHistoryDb.</p>
         * @param   root    the XML Document of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String getAction(Document doc, Permissions permit)
            throws SQLException, XMLException
        {
                return db().executeGetAction(doc.getDocumentElement());
        }

        /**
         * Gets the specified concept's edit actions in the date range.
         * <p>This method calls the corresponding method in org.nci.dts.db.ConceptHistoryDb.</p>
         * @param   root    the XML Document of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String getActionInDateRange(Document doc, Permissions permit)
            throws SQLException, XMLException
        {
                return db().executeGetActionInDateRange(doc.getDocumentElement());
        }

        /**
         * Gets the database information including the namespace id, the database name, the version
         * and the release date.
         * <p>This method calls the corresponding method in org.nci.dts.db.DbInfoDb.</p>
         * @param   root    the XML Document of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         * @throws XMLException
         * @since       DTS 2.4
         */
        public String getDbInfo(Document doc, Permissions permit)
            throws SQLException, XMLException
        {
                return dbinfo().executeGetDbInfo(doc.getDocumentElement());
        }

        /**
         * Executes the ping.
         * <p>This method calls the corresponding method in org.nci.dts.db.ConceptHistoryDb.</p>
         * @param   root    the XML Document of the query from the client side.
         * @return  the XML formatted result.
         * @throws SQLException
         */
        public String ping(Document doc, Permissions permit)
            throws Exception {

            String res = "";
            try{
            res = db().executePing(doc.getDocumentElement());
            } catch (SQLException sqlex){
                if( DbConnExceptionVerifier.getInstance().isBadConnectionException(sqlex) ){
                    isValidConnection = false;
                }
              return errorMarshaller().marshal(sqlex);
            }
            return res;

        }

        protected ErrorMarshaller errorMarshaller() {
          if(fErrorMarshaller == null) {
            fErrorMarshaller = new ErrorMarshaller();
          }
          return fErrorMarshaller;
        }

        protected boolean queryServerIsValid(){
               if( !isValidConnection )
                      Categories.dataServer().info( "### QueryServer [" + this + "] connection is no longer valid!" );
          return isValidConnection;
        }
        protected boolean isValidConnection = true;



        // accessors

  private ErrorMarshaller fErrorMarshaller;
  protected ConceptHistoryDb db() throws SQLException
        {
                if(fDb == null) { throw dbInitException(); }
                return fDb;
        }

        protected DbInfoDb dbinfo() throws SQLException
        {
                if(fDbInfo == null) { throw dbInitException(); }
                return fDbInfo;
        }

        protected void setDb(Connection c)
        {
                try
                {
                        fDb = new ConceptHistoryDb(c);
                        fDbInfo = new DbInfoDb(c);
                }
                catch(SQLException e)
                {
                        // store this exception for later use
                        setDbInitException(e);
                }
        }

        // A stored exception.  A non-null value indicates that the setDb() function failed.
        protected SQLException dbInitException() { return fDbInitException; }

        protected void setDbInitException(SQLException newDbInitException)
        {
                fDbInitException = newDbInitException;
        }
}